/* deo2ags  -  a translator / conversion program for 
               Alex Brown's .deo format to
                radware .ags level scheme format
First written   David Radford  April 2008
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <string.h>

#include "src/util.h"
#include "src/gls.h"

int main(int argc, char **argv)
{
  int   i;
  int   width_mode = 0;  /* 0 for BR, 1 for B(M1), 2 for B(E2) */
  int   odd_parity = 0;  /* 0 for even parity, 1 for odd parity levels */
  char  out_fn[256], *c, line[256];
  FILE  *infile, *outfile = 0;
  
  char *usage = 
    "\nUsage: deo2ags <input_file_name> [options]\n"
    "  Possible options: -n  (assume negative parity for all levels)\n"
    "                    -p  (assume negative parity for all levels, = default)\n"
    "                    -b  (use branching ratio for transition width, = default)\n"
    "                    -e  (use B(E2) for transition width)\n"
    "                    -m  (use B(M1) for transition width)\n"
    "                    -h  (help; generate this output)\n"
    "                    -o  <output_file_name>\n"
    "  (default output_file_name = input-file-name with .ags extension)\n";


  if (argc < 2 ||
      !(infile = fopen(argv[1], "r"))) {
    if (argc >= 2) printf("\nERROR: cannot open input file %s for reading...\n", argv[1]);
    printf(usage);
    return 1;
  }
  
  for (i=2; i<argc-2; i++) {  /* loop through input flags */
    if (strstr(argv[i], "-n")) {
      odd_parity = 1;
    } else if (strstr(argv[i], "-p")) {
      odd_parity = 0;
    } else if (strstr(argv[i], "-b")) {
      width_mode = 0;
    } else if (strstr(argv[i], "-m")) {
      width_mode = 1;
    } else if (strstr(argv[i], "-e")) {
      width_mode = 2;
    } else if (strstr(argv[i], "-o")) {
      if (++i >= argc-2) {
	printf("\nERROR: output file name is missing...\n");
	return 1;
      }
      if (!(outfile = fopen(argv[i], "w"))) {
	printf("\nERROR: cannot open output file %s for writing...\n", argv[i]);
	return 1;
      }
      strncpy(out_fn, argv[i], 255);
    } else {
      printf(usage);
      return 1;
    }
  }

  /* if output file name not already sepcitfied,
     usae the input file name with a .ags extension */
  if (!outfile) {
    strncpy(out_fn, argv[1], 250);
    i = strlen(out_fn) - 5;
    if ((c = strstr(out_fn + i, ".deo")) || (c = strstr(out_fn + i, ".DEO"))) {
      strcpy(c, ".ags");
    } else {
      strcat(out_fn, ".ags");
    }
    if (!(outfile = fopen(out_fn, "w"))) {
      printf("\nERROR: cannot open output file %s for writing...\n", out_fn);
      return 1;
    }
  }
  printf("\n  Input file: %s\n Output file: %s\n", argv[1], out_fn);

  return 0;
}
