C ****************************************************************************
C
C     Input (keyboard)
C     ****************
C     1.  Mode, Logon
C         Mode:
C         1 Automatic ENSDF->ENSDF2 conversion (nnn.ENS -> nnn.ENX)
C         2 Automatic ENSDF2->ENSDF conversion (nnn.ENX -> nnn.EN)
C         3 ENSDF->ENSDF2 conversion
C         4 ENSDF2->ENSDF conversion
C            Options 1-4 retain all information (all cards) of an
C            ENSDF or ENSDF/2 file.
C         5 ENSDF2->Band list (file list file)
C         6 Not used                  
C         7 ENSDF2->AGS format (A-chain conversion) Not implemented
C         8 ENSDF2->AGS format (file list file) Not implemented
C         The following options (where the first digit is the
C            read format and second digit is the write format)
C            only retains data that are saved in the common
C            blocks (see fconv.inc):
C            Format codes:
C            1. ENSDF
C            2. ENSDF/2
C            3. Radford's .AGS format
C            e.g.
C         12 ENSDF->ENSDF2 using common block data (EtoE2 does not write)
C         21 ENSDF2->ENSDF using common block data (E2toE does not write)
C         32 AGS->ENSDF2 format 
C         Logon: =0, errors printed to FCONV.LOG
C                =1, messages/errors printed to FCONV.LOG
C                =2, full printout to FCONV.LOG (for debugging)
C     2.  Input directory (current=blank)
C     3.  Output directory (current=blank)
C     4.  Data set selection string (e.g. DECAY, ADOPTED. Blank for
C         all data sets)
C     5.  For automatic ENSDF-ENSDF2 conversion: enter upper
C         and lower limit on mass number (missing files OK)
C         For manual conversion: enter input file name
C                                enter output file name
C         For manual conversion: Repeat 5 ad infinitum (blank to stop)
C
C     Input file: unit 1
C     Output file: unit 2
C     Log file: unit 4 (FCONV.LOG) contains a log of the conversion
C               procedures and errors encountered
C
C ***************************************************************************
C
C  Hints and comments
C  ******************
C
C  Radware AGS format
C
C  The mass number A is (deliberately) not defined in the AGS
C  format. Since A is needed in the NUCID field in the ENSDF format,
C  proceed as follows for AGS->ENSDF conversion:
C  1) Make sure that there is data for only one nuclide in the file
C  2) Add the following header as a comment among the first five lines:
C  ** AAAZZ 
C  e.g. 
C  ** 152Dy  
C
C  Unknown spins in AGS format
C  Non-band levels: spin=-1.0 or -0.5, spin label is -1 or -0
C  Band levels: band head spin -1 or -0.5, spin label -1 or -0
C               other levels 1.0,3.0... or 0.0,1.0... for even A
C                            1.5,3.5...    0.5,1.5... for odd A
C               unless given spins allow an extrapolation.
C  Spin sequence J+x: Treated as band levels with unknown band head spin.
C  If a value of J is given, it is used for a guess.
C
C ***************************************************************************
C
C  Revision log
C  ************
C
C    06/12/2000/dcr (4.0)
C           created by extracting relevant code from fconv.f

C ***************************************************************************

c     common blocks and variables for the program.

c     Parameters defining the size of arrays:
*      integer MLev,MGam,MBand,MCard

c  FConv variables:
*      integer logon
*      common /fconv/ logon

c  Dummy variables:
*      integer idum
*      double precision x(3)
*      common/dum1/ idum
*      common/dum2/ x
*      character dumstr*10
*      common/dums/ dumstr
*      character sdate*9,stime*8
*      common/dat/ sdate,stime

c  Data set variables:
*      integer A,Z,IOdd,DSStrL,LComm
*      real SP,DSP,SN,DSN,KRatio
*      logical Sep
*      common/dset/ A,Z,IODD,SP,DSP,SN,DSN,KRatio,Sep,DSStrL,LComm
c   A         Mass number of nucleus
c   Z         Charge of nucleus
c   IODD      =2 for odd nuclei, =1 for even nuclei
c   SP        Proton separation energy
c   DSP       Uncertainty in proton separation energy
c   SN        Neutron separation energy
c   DSN       Uncertainty in neutron separation energy
c   KRatio    Kinematic ratio for reaction (A(target)/A(comp.nucl.)
c   Sep       Separation (Sp or Sn) energy used in data set
c   DSStrL    Length of data set selection string
c   LComm     Length of conversion comment (character Comm(60))

c  Level variables:
*      parameter (MLev=3000)
*      parameter (MBand=50)
c   MLev      Maximum number of levels
c   MBand     Maximum number of bands
*      double precision EL(MLev),DEL(MLev)
*      real             T(MLev),DT(MLev),DT2(MLev)
*      integer nlev,ISCORE(MLev),L(MLev),LO(MLev),LOO(MLev),NDEC(MLev)
*      integer SPIN(MLev),BdStep(MBand),BdK(MBand),ioffs(MBand),Jlev
*      common/levels/ NLev,Jlev,EL,DEL,Iscore,L,LO,NDec,Spin,T,DT,
*     $               DT2,LOO,BdStep,BdK,ioffs
c   NLev      Number of levels
c   EL        Level energy
c   DEL       Uncertainty in level energy
c   IScore    Score for final level identification. High score is good.
c             ISCORE is a dummy variable used for finding the final level
c             in an ENSDF data set.
c   L         Level card of level i is card number L(i) (for ENSDF input)
c   LO        Array redefining level order
c   LOO       Array listing level numbers in energy order
c   NDec      Number of depopulating gammas from this level
c   Spin      Spin of level. J=2 is spin=2 (IODD=1). J=5/2 is spin=5 (IODD=2).
c             J=-1 is unknown spin
c   T         Half-life in seconds
c   DT        Right uncertainty in half-life
c   DT2       Left uncertainty in half-life
c   BdStep    Delta J in band
c   BdK       Spin of band head
c   ioffs     Spin offset of band
c

c  Gamma variables:
*      parameter (MGam=3000)
c   MGam      Maximum number of gammas
*      double precision egam(MGam),degam(MGam)
*      real    IGam(MGam),DIGam(MGam),D(MGam),DD(MGam),DD2(MGam)
*      integer ngam,inlev(MGam),finlev(MGam),ML(2,MGam),Jgam
*      common/gammas/ NGam,Jgam,EGam,DEGam,IGam,DIGam,
*     $               InLev,FinLev,D,DD,DD2,ML
c   EGam      Gamma-ray energy
c   DEGam     Uncertainty in gamma-ray energy
c   IGam      Gamma relative intensity
c   DIGam     Uncertainty in gamma relative intensity
c   InLev     Initial level of gamma (level number)
c   FinLev    Final level of gamma (level number)
c   D         Multipole mixing ratio
c   DD        Right uncertainty in multipole mixing ratio (+xx)
c   DD2       Left uncertainty in multipole mixing ratio (-xx)
c   ML        Multipolarities of transition (L-values). ML=-1 is unknown

c  Miscellaneous:
*      logical TheEnd,NewDS
*      integer NBand,NAlpha,NBeta,NPar
*      common/misc/ TheEnd,NewDS,NBand,NAlpha,NBeta,NPar
c   TheEnd    True when end of input file is reached
c   NewDS     True when new data set is found
c   NBand     Number of bands
c   NAlpha    Number of alphas
c   NBeta     Number of betas
c   NPar      Number of parents

c  Character common data:
*      parameter (MCard=4000)
*      character*1 MEl,Ban,BD,CARD*80,parity,DSign,MLPar,BdPi
*      character*2 egams,igams,ds,els,ts
*      character NUCID*5,DSID*71,BANDID*63,CPAR*5,MULT*10
*      character JPIMOD*3,MLMod*3,SNucId*5,DSStr*40,Comm*60
*      character*60 verstr,UncLev*1,UncTr*1,JOffs*1,BdStr*18,LevLab*10
*      common/char/ MEl(MLev),Ban(MLev),NUCID,DSID,BANDID(MBand),
*     $             BD(MBand),CPar(2),Parity(MLev),JPiMod(MLev),
*     $             els(MLev),ts(MLev),verstr,UncLev(MLev),JOffs(MLev),
*     $             BdStr(MBand),LevLab(MLev),BdPi(MBand)
*      common/char1/ Mult(MGam),EGamS(MGam),IGamS(MGam),DS(MGam),
*     $              DSign(MGam),MLMod(MGam),MLPar(MGam),SNucid,DSStr,
*     $              Comm,UncTr(MGam)
*      common/card1/ CARD(MCard)
c   MEL       Letter for +X in level energy
c   Ban       Band marker for level
c   NucId     Nucleus identification (A:1-3, element symbol:4-5)
c   DSId      Data set identification
c   BD        Band identifier
c   CPar      Parent Nucid (for decay)
c   Parity    Parity of level (values '+','-',' ' is unknown parity)
c   JPiMod    Modifier to SPIN and PARITY: ' (J' - spin uncertain,
c             ' (P' - parity uncertain, ' (B' - both uncertain.
c   MLPar     Parity of transition ('+','-',' ' is unknown)
c   ELS       Error string for Ex (contains 'GT','LT', etc.)
c   TS        Error string for half-life (contains 'GT','LT', etc.)
c   verstr    Version number of FConv
c   UncLev    Marker for uncertain levels     
c   JOffs     Offset for spin (e.g. J)
c   BdStr     JPi field of band head
c   LevLab    Level label (for LVL format)
c   BdPi      Parity of band (for labelling)
c   BandID    Band identification string
c   Mult      Multipolarity of gamma (ENSDF string)
c   EGamS     Error string for Egamma (contains 'GT','LT', etc.)
c   IGamS     Error string for Igamma (contains 'GT','LT', etc.)
c   DSign     Sign string for mixing ratio ('+' for positive MR,
c             ' ' for negative MR and for unknown sign)
c   DS        Error string for mixing ratio (contains 'GT','LT', etc.)
c   MLMod     Modifier to multipolarity
c             'xyz', x - first L, y - second L, z - parity. For
c             safe assignments x, y, z are ' ', for uncertain '(',
c             for assumed '[', for incorrect '*'.
c   SNucId    Nucid for which nucleon separation energies are given
c   DSStr     String for selecting data sets
c   Comm      String for exchanging commets between files
c   UncTr     Marker for uncertain transitions
c   Card      Card images of ENSDF data set (for ENSDF <-> ENSDF/2 conv.)
C ****************************************************************************

      implicit none

      character*70 inname,outname,form(0:3)*11,indir,outdir
      character*10 outSuff
      character*20 dattim

      integer i,ii,mode,mode1,mode2,indirl,outdirl,outSuffl
      integer lenstr,j
      integer num_per_nucl
      character*5 last_nucl
      character*2 fnsufsuf

      INCLUDE 'fconv.inc'

c$$
      data verstr /'ensdf2ags v0.0 00-06-12 '/
      data form /'NONE','ENSDF','ENSDF/2','Radware AGS'/

      LComm=0
c     Define normal order
      do i=1,mlev
         lo(i)=i
         loo(i)=i
      enddo

C  Unit 4: Log file
      OPEN(UNIT=4,FILE='ensdf2ags.log',STATUS='UNKNOWN')

      write(*,*) verstr
c120   write(*,*)'Mode:'
c	write(*,*)' 13 ENSDF->AGS using common block data'
c      write(*,*)'Logon =0, errors printed to FCONV.LOG'
c      write(*,*) ' Enter Mode, Logon:'
c      read(*,*) mode,logon
      mode  = 13
      logon = 0
      mode1 = mode/10
      mode2 = mode-10*mode1

      call datetime(dattim)
      sdate = dattim(1:9)
      stime = dattim(11:18)

      write(4,*) verstr
      write(4,*)' ',form(mode1),' -> ',form(mode2),' conversion'
      write(4,*)' LOGON=',logon
      write(4,*) ' Date/time: ',sdate,' ',stime
      write(4,*)
      write(4,*)'   Conventions for message classification:'
      write(4,*)'     *E* Errors '
      write(4,*)

c  Input Input and Output directories
      write(*,*)' Enter Input directory (Return for current directory)'
      read(*,'(A)') indir
      write(*,*)' Enter Output directory (Return for current directory)'
      read(*,'(A)') outdir
      call tbsup(indir)
      call tbsup(outdir)
      indirl=lenstr(indir)
      outdirl=lenstr(outdir)
      write(4,*) 'Input directory: ',indir
      write(4,*) 'Output directory: ',outdir

c     Input data set selection string
      write(*,*) ' Enter data set selection string (Case is '
      write(*,*) ' significant, CR for all data sets):'
      read(*,'(A)') DSStr
      call tbsup(DSStr)
      DSStrL=lenstr(DSStr)
	if(DSStrL.gt.0) then
		write(*,*) 'Data set selection string:'
		write(*,*) '"'//DSStr(1:DSStrL)//'"'
		write(4,*) 'Data set selection string:'
		write(4,*) '"'//DSStr(1:DSStrL)//'"'
	endif

c  Manual ENSDF/ENSDF2 -> AGS conversion using common block data
c      elseif (mode.ge.10 .and. (mode1.le.2.and.mode2.eq.3)) then
51       continue

         last_nucl = 'xxxxx'
         write(*,*) form(mode1),' -> ',form(mode2),' conversion'

         write(*,*) ' Input filename (Blank=STOP):'
         read(*,'(A)') inname
         if(inname.eq.' ') goto 99
         if(mode2.gt.0) then
            write(*,*) ' Suffix for output filename:'
            read(*,'(A)') outSuff
		  call tbsup(outSuff)
		  outSuffl=lenstr(outSuff)
         endif

C        Unit 1: Input file
         if(indirl.gt.0) then
            OPEN(UNIT=1,FILE=indir(1:indirl)//inname,STATUS='OLD')
         else
            OPEN(UNIT=1,FILE=inname,STATUS='OLD')
         endif

         TheEnd=.FALSE.

         do i=1,9000
c           Read data
            CALL EtoE2(1,0)

            if (NLev.ge.1 .and.
     +          (outSuff(1:4).ne.'misc' .or.
     +           (index(DSID,'(HI,XNG)').eq.0 .and.
     +            index(DSID,'(POL').eq.0 .and.
     +            index(DSID,'(PI').eq.0 .and.
     +            index(DSID,'(K').eq.0 .and.
     +            index(DSID,'(MU').eq.0 .and.
     +            index(DSID,'(G,').eq.0 .and.
     +            index(DSID,'(N,').eq.0 .and.
     +            index(DSID,'(P,').eq.0 .and.
     +            index(DSID,'(D,').eq.0 .and.
     +            index(DSID,'(T,').eq.0 ))) then

               if (NUCID.eq.last_nucl) then
                  num_per_nucl = num_per_nucl + 1
               else
                  num_per_nucl = 0
               endif
               last_nucl = NUCID

c              Write data
               ii = 3
               do while (ii.lt.5 .and.
     +                   NUCID(ii+1:ii+1).ne.' ' .and.
     +                   NUCID(ii+1:ii+1).ne.char(0))
                   ii = ii+1
               enddo

               if(outSuff(1:4).eq.'misc') then
                  write(fnsufsuf,'(i2.2)') num_per_nucl+1
                  outname = NUCID(1:3)//'/'
     +                 //NUCID(1:ii)//'_ensdf_'
     +                 //outSuff(1:outSuffl)
     +                 //fnsufsuf//'.ags'
               elseif(outSuffl.gt.0) then
                  if (num_per_nucl.gt.0) then
                     write(fnsufsuf,'(i2.2)') num_per_nucl+1
                     outname = NUCID(1:3)//'/'
     +                             //NUCID(1:ii)//'_ensdf_'
     +                             //outSuff(1:outSuffl)
     +                             //fnsufsuf//'.ags'
                  else
                     outname = NUCID(1:3)//'/'
     +                             //NUCID(1:ii)//'_ensdf_'
     +                             //outSuff(1:outSuffl)//'.ags'
                  endif
               else
                  outname = NUCID(1:ii)//'_ensdf.ags'
               endif
               if (outname(1:2).eq.'  ') then
                  outname(1:2) = '00'
                  outname(5:) = outname(7:)
               elseif (outname(1:1).eq.' ') then
                  outname(1:1) = '0'
                  outname(5:) = outname(6:)
               endif
               include 'open.inc'

               if (NewDS) call WAGS(2,mode)
c              Print data to log file

c              Check data
               call CheckDS
c              Show progress on screen
               if(NewDS) write(*,*) NUCID,'  ',DSID(1:71)
               if(TheEnd) goto 61
            endif
         enddo
61       continue
         close(1)
         close(2)
         close(22)
         goto 51


99    continue
      STOP
      END

      SUBROUTINE CheckDS

      implicit none
      INCLUDE 'fconv.inc'
      integer i,ii,iii,iiold,j,jj,jjj,prev,next,i1
      double precision eold

      iiold=0
      eold=0

      do i=1,ngam
         ii=inlev(i)
         iii=finlev(i)
C        Determine previous/next final level
         prev=0
         next=NLev
         if(iii.gt.1) prev=iii-1
         if(iii.lt.NLev) next=iii+1
         i1=i+1
         do j=i1,ngam
            jj=inlev(j)
            jjj=finlev(j)

c           Check for doubly matched gammas
            if(ii.eq.jj.and.iii.eq.jjj) then
               write(4,100)'*E*',ii,mel(ii),Ban(ii),el(ii),iii,mel(iii),
     $                      Ban(iii),el(iii),egam(i),egam(j)
100            format(/1X,A' Doubly matched gammas from level',
     $          I5,1X,A,1X,A,F8.2,' to level',I5,1X,A,1X,A,F8.2,
     $          /2X,F10.2,F10.2)
            write(4,103) el(prev),del(prev),el(next),del(next)
            endif
         enddo

c        Check that gamma fits between levels
         call chgam(el(ii),del(ii),el(iii),del(iii),egam(i),degam(i),A,
     $       el(prev),del(prev),el(next),del(next),logon)

c        Check gamma multipolarity against spin/parity
         call chmult(Spin(ii), Parity(ii), JPiMod(ii),
     $               Spin(iii),Parity(iii),JPiMod(iii),
     $               ML(1,i),  MLPar(i),   MLMod(i),mult(i),Iodd,
     $               egam(i),el(ii),el(iii),d(i),dd(i),dd2(i),
     $               el(prev),del(prev),el(next),del(next),logon)

c        Check gamma ordering
         if(egam(i).lt.eold.and.ii.eq.iiold) then
            if(logon.gt.0) write(4,101) '*W*',el(ii),eold,egam(i)
101         format(/1X,A,' Gammas in the wrong order, level=',F10.2,
     $             '. Gammas:',2F10.2)

         endif
         iiold=ii
         eold=egam(i)

c        Make sure levels have the same MEL letter
         if(MEL(ii).ne.MEL(iii).and.logon.gt.0) then
            write(4,201) '*W*',egam(i),mel(ii),el(ii),mel(iii),el(iii)
201         format(/1X,A,' Inconsistent final level:',
     $           /'     Gamma: ',F8.2,'. Initial level: 'A,1X,F8.2,
     $            ', final level: ',A,1X,F8.2)
            write(4,103) el(prev),del(prev),el(next),del(next)
103         format('     Previous/next final level:'2F10.3,' and',
     $            2F10.3)
         endif
      enddo

      return
      END

      SUBROUTINE EtoE2(INP,IOUT)
c  Reads ENSDF and writes ENSDF/2

      implicit none
      DOUBLE PRECISION etemp,dle,detemp,fmax,deg,demin
      logical SkipLevel
      integer i,ng,inp,iout,nn,ij,jj,numb,j
      integer nerr,max,nscore,nmat,nk,k,ik,itypes
c     Character strings for subroutine FEND:
      CHARACTER*2 NE*3
c     Local character strings:
      character*5 MLL,MGG,GMATCH*10,unit*3
c     Local card character strings
      character*80 ICARD
      character lflag(200)*1
      integer nlflag

      INCLUDE 'fconv.inc'

      nlflag=0
      NLEV=0
      NG=0
      NGAM=0
      NALPHA=0
      NBETA=0
      NPAR=0
      NBAND=0
      CPAR(1)='     '
      CPAR(2)='     '
      NewDS=.true.
      Sep=.false.

c  Read dataset
c  ************
98    NN=0
c     Loop through all cards of a data set
      do 500 i=1,50000
         NN=NN+1
         if(NN.GT.MCARD) call stopit(card(1),
     $    ' ** Too many cards **',' Increase MCARD',' ',NN)
         read(INP,'(A)',end=99) card(NN)

c        * For debugging *
c         write(*,'(1X,I4,'' '',A)') NN,card(NN)(1:70)

c        Skip blank cards
         if(i.eq.1.and.card(1).eq.' ') goto 98
         GOTO 101

c        End-of-file found
99       TheEnd=.TRUE.
         NN=NN-1
c        Take care of ending blank card
         if(NN.LE.1) NewDS=.false.
         go to 5000

101      continue
c  Find out what type of card it is
c  Data set identification
c  -----------------------
         IF(NN.eq.1) THEN
            if(card(NN)(6:9).NE.'   ') call stopit
     $         (card(1),' ** Invalid ID card **',' ',' ',0)

		  if(DSStrL.gt.0) then
			 if(index(card(1),DSStr(1:DSStrL)).eq.0) then
                  call skip(INP)
                  if(TheEnd) goto 99
                  goto 98
               endif
		  endif

            write(4,'(/,a)') card(NN)(1:80)
            card(NN)(9:9)='*'
            DSID=card(NN)(10:80)

c           Save NUCID. Get A and Z. Determine IODD.
            CALL ZNAME (card(NN)(4:5),Z,1)
            NUCID=card(NN)(1:5)
            READ (NUCID(1:3),'(I3)') A
            CALL ODDEVN (A,IODD)

c  Level card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  L ') THEN
            NLEV=NLEV+1
            if(NLEV.GT.MLEV) call stopit(card(1),
     $       ' ** Too many levels **',' Increase MLEV',' ',NLEV)
c           Save band head spin if first level in band
            do idum=1,NBand
               if(card(NN)(77:77).eq.BD(idum)) then
                  if(BdStr(idum).eq.' ') then
                     if(card(NN)(22:39).ne.' ') then
                        BdStr(idum)=card(NN)(22:39)
                     else
                        BdStr(idum)='?'
                     endif
                  endif
               endif
            enddo
c           Get energy
            CALL FEND(card(NN)(10:19),card(NN)(20:21),x,NE,els(NLEV))
            EL(NLEV)=x(1)
            DEL(NLEV)=x(2)
            MEL(NLEV)=' '
            Ban(NLEV)=' '

c           Calculate level energy if given as SP+x or SN+x
            call SpSn(card(NN)(10:19),SkipLevel)
            if(SkipLevel.and.IOUT.eq.0) then
               NLEV=NLEV-1
               goto 515
            endif

c           Save spin/parity and half-life, reset NDEC
            call GetJPi(card(NN)(22:39),spin(NLEV),parity(NLEV),
     $       jpimod(NLEV),JOffs(NLev))
            CALL FENDU(card(NN)(40:49),card(NN)(50:55),x,
     $       NE,ts(NLEV),unit)
            call un2sec(x,unit,ts(NLEV))
            t(NLEV)=x(1)
            dt(NLEV)=x(2)
            dt2(NLEV)=x(3)
            NDec(NLev)=0

c           Check level order
            IF(NLEV.GT.1.AND.EL(NLEV).LT.EL(NLEV-1).AND.EL(NLEV)
     $       .GE.1.0E-10) THEN
c              Allow E+X levels to be "out of order"
               IF(INDEX(card(NN)(10:19),'+').LE.0.AND.
     $          INDEX(card(L(NLEV-1))(10:19),'+').LE.0) THEN
                  call stopit(card(1),card(NN),card(L(NLEV-1)),
     $                        ' ** Level out of order **',0)
               ENDIF
            ENDIF
c           Save card number for later use
            L(NLEV)=NN

c           Left justify for FL= matching
            DO IJ=10,19
               IF(card(L(NLEV))(10:10).NE.' ') GO TO 513
               card(L(NLEV))(10:18)=card(L(NLEV))(11:19)
               card(L(NLEV))(19:19)=' '
            ENDDO
513         CONTINUE

c           +X goes to MEL array
            JJ=INDEX(card(NN)(10:19),'+')+9
c           Check for exponent, SP or SN
            if(JJ.GT.9.AND.card(NN)(JJ-1:JJ-1).NE.'E') then
               if(index(card(NN),'SP').le.0.and.
     $          index(card(NN),'SN').le.0) then
c                 Check for X+
                  IF(NUMB(card(NN)(JJ-1:JJ-1)).EQ.0.AND.card(NN)
     $             (JJ-1:JJ-1).NE.' ') MEL(NLEV)=card(NN)(JJ-1:JJ-1)
c                 Check for +X
                  IF(NUMB(card(NN)(JJ+1:JJ+1)).EQ.0.AND.card(NN)
     $             (JJ+1:JJ+1).NE.' ') MEL(NLEV)=card(NN)(JJ+1:JJ+1)
               endif
c              Check for letter by itself
            elseif(ITYPES(card(NN)(10:19)).ne.-2) then
               dumstr=card(NN)(10:19)
               call lbsup(dumstr)
               mel(NLEV)=dumstr(1:1)
            endif

c           Put col 80 (?) to col 1, col 78 (M) to col 2 and clear structure
c           flag (col. 3)
            UncLev(NLev)=card(NN)(80:80)
            card(NN)(1:1)=card(NN)(80:80)
            card(NN)(2:2)=card(NN)(78:78)
            card(NN)(78:80)='   '
            card(NN)(3:3)=' '
c           Add level ID to cols 4-5
            CALL CHARAD(NLEV,card(NN)(4:5))

c           Set stucture flag in col. 3 and other flags on FLAG= cont. card
            IF(card(NN)(77:77).NE.' ') THEN
c              Check if band with col. 77 tag is defined
               do 514 ij=1,NBAND
                  IF(BD(ij).NE.card(NN)(77:77)) GOTO 514
                  card(NN)(3:3)=BD(ij)
                  card(NN)(77:77)=' '
                  Ban(NLEV)=BD(IJ)
c                 If there is a level comment with the same tag, write
c                  a FLAG= card
                  do jj=1,nlflag
                     if (lflag(jj).eq.Ban(NLev)) then
                        NN=NN+1
                        card(NN)=NUCID//'F L FLAG='//card(NN-1)(3:3)
                        goto 515
                     endif
                  enddo
                  go to 515
514            continue
c              It is not a band; put it on FLAG= cont.card
               NN=NN+1
               card(NN)=NUCID//'F L FLAG='//card(NN-1)(77:77)
               card(NN-1)(77:77)=' '
            ENDIF
515         continue

c  Alpha card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  A ') THEN
            NALPHA=NALPHA+1
            card(NN)(1:5)='     '

c  EC/Beta+ card
c  -------------
         ELSEIF(card(NN)(6:9).EQ.'  E ') THEN
            NBETA=NBETA+1
            card(NN)(1:5)='     '

c  Beta- card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  B ') THEN
            NBETA=NBETA+1
            card(NN)(1:5)='     '

c  Parent card
c  -----------
         ELSEIF(card(NN)(6:9).EQ.'  P ') THEN
            NPAR=NPAR+1
c           Put parent Nucid in CPAR
            IF(NPar.LE.2) CPar(NPar)=card(NN)(1:5)

c  Q card
c  ------
         ELSEIF(card(NN)(6:9).EQ.'  Q ') THEN
c           Put Nucid in SNucId
            CALL ZNAME (card(NN)(4:5),idum,1)
            SNucId=card(NN)(1:5)
c           Save SP and SN with uncertainties
            CALL FEND(card(NN)(22:29),card(NN)(30:31),x,NE,dumstr)
            Sn=x(1)
            DSn=x(2)
            CALL FEND(card(NN)(32:39),card(NN)(40:41),x,NE,dumstr)
            Sp=x(1)
            DSp=x(2)

c  End card
c  --------
         ELSEIF(card(NN).EQ.' ') THEN
            GO TO 5000

c  Gamma card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  G ') THEN
c           Clear NUCID field. Move ? flag to col. 1. Clear coin field.
            card(NN)(1:5)='     '
            NERR=0
            card(NN)(1:1)=card(NN)(80:80)
            card(NN)(78:80)='   '

c           Ignore unplaced gammas and decay from levels with E=0
            IF(NLEV.EQ.0) GO TO 508
            if(el(NLEV).le.0) then
               write(4,211) card(NN)(10:19),card(L(NLEV))(10:19)
211            format(/'*E* Gamma depopulating level with E=0'
     $                /' Gamma ',A,' from level ',A,' ignored')
               go to 508
            endif

c           Band assignment code from initial level to gamma record col. 2
            card(NN)(2:2)=card(L(NLEV))(3:3)
            NG=NN
            NGAM=NGAM+1
            if(NGAM.GT.MGAM) call stopit(card(1),
     $       ' ** Too many gammas **',' Increase MGAM',' ',NGAM)
            inlev(ngam)=NLEV

            UncTr(NGam)=card(NN)(1:1)
c           Get energy, default DE for matching is 1 keV
            CALL fend(card(NN)(10:19),card(NN)(20:21),x,NE,egams(NGAM))
            egam(NGAM)=x(1)
            degam(NGAM)=x(2)
            DEG=degam(NGAM)
            IF(degam(NGAM).LT.1.0D-10) DEG=1.0
            ETEMP=EL(NLEV)-egam(NGAM)
            DLE=DEL(NLEV)
            IF(DLE.LT.1.0D-10) DLE=1.0
            DETEMP=DSQRT(DEG**2+DLE**2)
c           Count the gammas from this level
            NDEC(NLEV)=NDEC(NLEV)+1

c           Get intensity
            CALL fend(card(NN)(22:29),card(NN)(30:31),x,NE,igams(NGAM))
            igam(NGAM)=x(1)
            digam(NGAM)=x(2)

c           Get multipolarity
            MULT(NGAM)=card(NN)(32:41)
            call GetML(mult(NGam),ML(1,NGam),MLPar(NGam),MLMod(NGam))

c           Get mixing ratio
            ds(NGam)='  '
            CALL fend(card(NN)(42:49),card(NN)(50:55),x,NE,ds(NGAM))
            d(NGAM)=x(1)
            dd(NGAM)=x(2)
            dd2(NGAM)=x(3)
c           If + sign is given, save it in DSign
            DSign(NGam)=' '
            dumstr=card(NN)(42:49)
            call lbsup(dumstr)
            if(dumstr(1:1).eq.'+')  DSign(NGam)='+'

            MAX=1
            NSCORE=0
            DEMIN=1000.0
            NMAT=0

c           Score match on basis of energy, Mult-JPi, band assignment, and +XYZ
            DO 541 IJ=1,NLEV-1
               ISCORE(IJ)=0
c              Test only cases where +XYZ are identical or absent
               IF(MEL(IJ).NE.MEL(NLEV)) GO TO 541
               CALL ESCORE(ETEMP,DETEMP,EL(IJ),DEL(IJ),ISCORE(IJ))

c              If no energy match give up and try again
               IF(ISCORE(IJ).EQ.0) GO TO 541

c              Check for common band assignment, add 2 to score
               IF(card(L(NLEV))(3:3).EQ.card(L(IJ))(3:3).AND.
     $          card(L(NLEV))(3:3).NE.' ') ISCORE(IJ)=ISCORE(IJ)+2

c              Check for consistent multipolarity and JPI:s
               CALL MJSCOR(card(NN)(32:41),card(L(NLEV))(22:39),
     $          card(L(IJ))(22:39),IODD,ISCORE(IJ))
               IF(ISCORE(IJ).LE.0) GO TO 541
               NMAT=NMAT+1

c              MAX is level with best fit (highest score)
               IF(ISCORE(IJ).LT.NSCORE) GO TO 541
c              Two levels with same score, choose closest
               IF(ISCORE(IJ).EQ.NSCORE) THEN
                  IF(abs(ETEMP-EL(IJ)).lt.DEMIN) THEN
                     MAX=IJ
                     DEMIN=abs(ETEMP-EL(MAX))
                     NSCORE=ISCORE(IJ)
                  ENDIF
c              Two levels with different score, choose best score
               ELSE
                  MAX=IJ
                  DEMIN=abs(ETEMP-EL(MAX))
                  NSCORE=ISCORE(IJ)
               ENDIF
541         CONTINUE

            if(nscore.gt.0) then
               card(NN)(4:5)=card(l(max))(4:5)
               card(NN)(3:3)=Ban(max)
               finlev(ngam)=MAX
               GO TO 542
            endif

c           If no fits get a score, write to output file for further review and
c           choose closest level
            CALL MATCH(ETEMP,EL,NLEV,MAX)
c           Make sure final level is not initial level
            if(MAX.EQ.NLEV) MAX=MAX0(NLEV-1,1)
            card(NN)(4:5)=card(L(MAX))(4:5)
            card(NN)(3:3)=Ban(max)
            finlev(ngam)=MAX

C           If closest level is only reasonable fit, all is OK
            FMAX=5.0*DETEMP
            IF(FMAX.LT.5.0) FMAX=5.0
            IF(MAX.EQ.1.AND.EL(MAX+1)-EL(MAX).GT.FMAX) GO TO 542
            IF(MAX.GT.1.AND.EL(MAX+1)-EL(MAX).GT.FMAX.AND.EL(MAX)-
     $       EL(MAX-1).GT.FMAX) GO TO 542
            if(logon.ge.1) then
               WRITE(4,255) card(NN)(10:49),card(L(NLEV))(3:3),
     $          card(L(NLEV))(10:49),ETEMP
               WRITE(4,256) card(L(MAX))(3:3),card(L(MAX))(10:56)
256            FORMAT(6X,'# Matched to level  ',A,' ',A)
            endif
            NERR=1

            if(logon.ge.1) then
               IF(MAX.EQ.1) WRITE(4,257) card(L(2))(3:3),
     $          card(L(2))(10:56)
               IF(MAX.GT.1) WRITE(4,257)
     $          card(L(MAX-1))(3:3),card(L(MAX-1))(10:56),
     $          card(L(MAX+1))(3:3),card(L(MAX+1))(10:56)
257            FORMAT(7X,' Nearby levels are ',A,' ',A,
     $               /7X,' and               ',A,' ',A)
            endif
            GO TO 511

542         CONTINUE

            card(NN)(4:5)=card(L(MAX))(4:5)
            card(NN)(3:3)=Ban(max)
            finlev(ngam)=MAX

C           If multiple possibilities write all to output
            IF(NMAT.LE.1) GO TO 511
            if(logon.ge.1) then
               WRITE(4,255) card(NN)(10:49),card(L(NLEV))(3:3),
     $          card(L(NLEV))(10:49),ETEMP
255            FORMAT(/' Gamma with E=',A
     $                /'  from ',A,' level with E=',A,' DeltaE=',F8.2)
            endif
            DO IJ=1,NLEV-1
               IF(ISCORE(IJ).GT.0.and.logon.ge.1) THEN
                  WRITE(4,407) card(L(IJ))(3:3),
     $                         card(L(IJ))(10:49),ISCORE(IJ)
407               FORMAT(14X,A,' level=',A,7X,'Score=',I3)
                  NERR=1
               ENDIF
            ENDDO
            if(logon.ge.1) WRITE(4,'(''      # Level chosen: '',A)')
     $       card(L(MAX))(10:49)

511         CONTINUE

508         CONTINUE

c           Put col. 77 flag on FLAG= cont.card
            if(card(NN)(77:77).ne.' ') then
               NN=NN+1
               card(NN)='     '//'F G FLAG='//card(NN-1)(77:77)
               card(NN-1)(77:77)=' '
            endif

c  Final level flag (FL=) on gamma continuation card
c  -------------------------------------------
      ELSEIF(card(NN)(7:9).EQ.' G '.AND.INDEX(card(NN),'FL=').GT.0) THEN
         JJ=INDEX(card(NN),'FL=')+2
         IF(JJ.LE.2) GO TO 512
         GMATCH='          '
         NK=0
         DO K=1,12
c           Skip blanks. Put FL value left justified in GMATCH
            IF(card(NN)(JJ+K:JJ+K).NE.' ') THEN
               IF(card(NN)(JJ+K:JJ+K).EQ.'$') GO TO 523
               NK=NK+1
               GMATCH(NK:NK)=card(NN)(JJ+K:JJ+K)
            ENDIF
         ENDDO
523      CONTINUE

c        Compare previous levels with GMATCH
         DO IJ=1,NLEV-1
            IF(card(L(IJ))(10:19).EQ.GMATCH) THEN
c              Match found
               card(NG)(4:5)=card(L(IJ))(4:5)
               card(NN)(3:3)=Ban(IJ)
               finlev(ngam)=IJ
c              Do not output FL= second card
               NN=NN-1
               IF(NERR.EQ.1) THEN
                  if(logon.ge.1) WRITE(4,259) GMATCH
259               FORMAT('  & Problem resolved by FL=',A)
               ENDIF
               GO TO 512
            ENDIF
         ENDDO
c        Error message for no match found
         WRITE(4,1525) '*E*',GMATCH,card(NG)(1:22)
1525     FORMAT(/1X,A,' No FL=',A,' match for ',A)
c        Do not output FL= second card
         NN=NN-1

512      CONTINUE

c  Band (level) comment
c  --------------------
      ELSEIF((card(NN)(6:9).EQ.' CL '.OR.card(NN)(6:9).eq.' cL ')
     $ .AND.INDEX(card(NN)(10:20),'BAND(').GT.0) THEN
         JJ=INDEX(card(NN),'BAND(')+4
         IF(JJ.LE.4) GO TO 510

         do j=1,10
            IF(card(NN)(JJ+j:JJ+j).EQ.')'.or.
     $       card(NN)(JJ+j:JJ+j).EQ.' ') GOTO 510

            if(card(NN)(JJ+j:JJ+j).ne.',') then
               NBAND=NBAND+1
               IF(NBAND.GT.MBAND) call stopit(card(1),
     $          ' ** Too many bands **',' Increase MBAND',' ',NBAND)
               BD(NBAND)=card(NN)(JJ+j:JJ+j)
c              Clear band head JPi field
               BdStr(NBand)=' '

c              Clear col:s 1-5 and put band flag in col. 3
               card(NN)(1:5)='     '
               card(NN)(3:3)=BD(NBAND)

c              Save comment text as band identification
               IJ=INDEX(card(NN),'$')+1
               IF(IJ.le.1.or.IJ.gt.20) IJ=20
               BANDID(NBAND)=card(NN)(IJ:80)
            endif
         enddo

510      CONTINUE

c  Band flag on level continuation card (FLAG=)
c  --------------------------------------------
         ELSEIF(card(NN)(7:9).EQ.' L '.AND.INDEX(card(NN),'FLAG=').GT.0)
     $    THEN
            IF(card(L(NLEV))(3:3).NE.' ') go to 509
            JJ=INDEX(card(NN),'FLAG=')+3
            IF(JJ.LE.3) GO TO 509
            DO IJ=2,10
               IF(card(NN)(JJ+IJ:JJ+IJ).EQ.'$') GO TO 509
               IF(card(NN)(JJ+IJ:JJ+IJ).NE.' '.AND.
     $          card(NN)(JJ+IJ:JJ+IJ).NE.',') THEN
                  DO IK=1,NBAND
                     IF(BD(IK).EQ.card(NN)(JJ+IJ:JJ+IJ)) THEN
                        card(L(NLEV))(3:3)=BD(IK)
                        Ban(NLEV)=BD(IK)
c                       Save band head spin if first level in band
                        if(BdStr(IK).eq.' ') then
                           if(card(L(NLev))(22:39).ne.' ') then
                              BdStr(IK)=card(L(NLev))(22:39)
                           else
                              BdStr(IK)='?'
                           endif
                        endif
                        GO TO 509
                     ENDIF
                  ENDDO
               ENDIF
            ENDDO

509         CONTINUE

c  Level comment which is NOT a band comment
c  -----------------------------------------
         ELSEIF((card(NN)(6:9).EQ.' CL '.OR.card(NN)(6:9).eq.' cL ')
     $    .AND.INDEX(card(NN)(10:20),'(').GT.0) THEN
            JJ=INDEX(card(NN),'(')+1
            nlflag=nlflag+1
            lflag(nlflag)=card(NN)(JJ:JJ)
            card(NN)(1:5)='     '

c  Remove nucid on any other card
c  ------------------------------
         ELSE
            card(NN)(1:5)='     '
         ENDIF

500   CONTINUE

c  Prepare and write data to ENSDF/2 file
c  **************************************
5000  CONTINUE
      IF(.NOT.NewDS) RETURN

c     Do not write ENSDF/2 data set if IOUT is 0
      IF(IOUT.EQ.0) RETURN

c     Write ID card
      WRITE(IOUT,'(A)') card(1)

c     Prepare information (I) card
      ICARD=' '
      IF(card(1)(4:5).NE.'  ') THEN
         Icard(1:3)=card(1)(1:3)
         WRITE(Icard(4:6),'(I3)') Z
         Icard(8:8)='I'
         WRITE(Icard(11:14),'(I4)') NLEV
         WRITE(Icard(21:24),'(I4)') NGAM
         WRITE(Icard(31:34),'(I4)') NALPHA
         WRITE(Icard(41:44),'(I4)') NBETA
         WRITE(Icard(51:54),'(I4)') NPAR
         Icard(55:59)=CPAR(1)
         Icard(60:64)=CPAR(2)
         WRITE(Icard(71:74),'(I4)') NBAND
         WRITE(IOUT,'(A)') Icard(1:74)
      ENDIF

      MLL='     '
      MGG='     '
      DO IJ=2,NN
         IF(card(IJ)(6:9).EQ.'  L ') MLL=card(IJ)(1:5)
         IF(card(IJ)(6:9).EQ.'  G ') MGG=card(IJ)(1:5)

c  Find last non-blank character
         DO IK=1,80
            IF(card(IJ)(81-IK:81-IK).NE.' ') GO TO 5003
         ENDDO
5003     CONTINUE
         IK=81-IK

c  Make sure level/gamma comments and cont. cards have level and band info
         IF(card(IJ)(8:8).EQ.'L'.and.MLL.ne.' ') card(IJ)(1:5)=MLL
         IF(card(IJ)(8:8).EQ.'G'.and.MGG.ne.' ') card(IJ)(1:5)=MGG

c  Write cards to output
         if(IK.gt.0) then
            WRITE(IOUT,'(A)') card(IJ)(1:IK)
         else
            WRITE(IOUT,'(A)') ' '
         endif

      ENDDO
      return
      END


      SUBROUTINE ODDEVN(A,IODD)
C   DETERMINES IF ENSDF DATASET HAS ODD-A OR EVEN-A
      implicit none
      integer A,IODD,NA
C     A odd: NA=1 IODD=2
C     A even: NA=0 IODD=1

      IODD=2
      NA=(A+1)/2-A/2
      IF(NA.EQ.0) IODD=1
      RETURN
      END


      SUBROUTINE ESCORE(E,DE,X,DX,ISCORE)
c  Decide on final level for gamma
c  Score energy. Small uncertainty, small deviation adds 9 to score

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

c     Calculate uncertainty in deviation
      DZ=(DE**2+DX**2)**0.5
      IF(DZ.LT.0.0001) DZ=1.0
      MEZ=0
      JSCORE=0
c     Z is deviation in sigmas
      Z=DABS((E-X)/DZ)

c Relax constraint for precise energy matches
      if(dabs(e-x).lt.0.03.and.z.gt.1.0) z=1.0

c??  Can this happen?
      if(iscore.NE.0) then
         if(z.gt.6) return
         if(z.gt.1) z=1.5
         stop 9999
c         go to 521
      endif

      IF(DZ.LE.4.0) GO TO 521
      IF(DZ.GE.10.0) GO TO 524
      MEZ=2

c       DZ is 4-10. No score if Z > 2 sigma
      IF(Z.GT.3.0) RETURN
      IF(Z.GT.2.0) GO TO 520
      GO TO 522

c       DZ is >10. No score if Z > 2 sigma
524   IF(Z.GT.2.0) RETURN
      MEZ=4
      GO TO 523

c       DZ is 0-4. No score if Z > 3 sigma
521   CONTINUE
      IF(Z.GT.4.0) RETURN
      IF(Z.GT.3.0) GO TO 520
      IF(Z.LT.3.0) JSCORE=2
522   IF(Z.LT.2.0) JSCORE=5-MEZ
523   IF(Z.LT.1.0) JSCORE=7-MEZ
      IF(Z.LT.0.3) JSCORE=9-MEZ
520   CONTINUE

c     Add energy score to ISCORE and return
      ISCORE=ISCORE+JSCORE

      RETURN
      END

      SUBROUTINE MJSCOR(MULL,MJ1,MJ2,IODD,ISCORE)
C  This subroutine matches multipolarity to initial and final spins
C  Score 0 for partial match, 7 for complete match and -7 for mismatch

      CHARACTER*1 JPAR(5),KPAR(5),IERR
      CHARACTER*2 MEL(15)
      CHARACTER*3 MJP(5)
      CHARACTER*10 MUL,MULL
      CHARACTER*18 MJ1,MJ2,IBLANK
      DIMENSION JSPIN(5),KSPIN(5)

      DATA MEL/'E1','M2','E3','M4','E5','E0','M1','E2','M3','E4','M5',
     2'E6','D ','Q ','O '/

      IBLANK='                  '
c     No score if any of the spin fields is empty
      IF(MJ1.EQ.IBLANK.OR.MJ2.EQ.IBLANK) RETURN

      MUL=MULL
c     Treat no multipolarity as Quadrupole
      IF(MUL.EQ.'          ') MUL='  Q       '

c     No score if multipolarity is based on spins (IF or [])
      IF(INDEX(MUL,'IF').NE.0.OR.INDEX(MUL,'[').NE.0) RETURN
      JT=-1

c     Is multipolarity dipole (D)? This takes care of D+Q
      IF(INDEX(MUL,'D').NE.0) THEN
         L=13
         GO TO 11
      ENDIF

      DO 10 L=1,15
         LL=2
         IF(LL.GT.12) LL=1
         IF(INDEX(MUL,MEL(L)(1:LL)).NE.0) GO TO 11
10    CONTINUE

      JT=0
c     No match found, return
      IF(L.EQ.15) RETURN

11    CONTINUE
      LDQ=0
c     Dipole/quadrupole/octupole
      IF(L.GT.12) LDQ=1
      IF(L.EQ.13) L=1
      IF(L.EQ.14) L=2
      IF(L.EQ.15) L=3

      IF(L.LE.5) GO TO 12
      L=L-6
      JT=1

c     JT=1 for no parity change. JT=0 for parity change. L is multipolarity
12    CONTINUE
c     Double multipolarity to handle half-integer spins
      L=L*IODD
      NMAT=0

C  MJ1 and MJ2 are the inital and final level spins respectively
      CALL JREAD(MJ1,JSPIN,JPAR,MJP,N1,IERR)
      CALL JREAD(MJ2,KSPIN,KPAR,MJP,N2,IERR)

C  PROCESS  D, Q, O
      IF(LDQ.NE.0) THEN
         DO 30 I=1,N1
            DO 31 J=1,N2
               IF(IABS(JSPIN(I)-KSPIN(J)).GT.L) GO TO 31
               NMAT=NMAT+1
31          CONTINUE
30       CONTINUE
      GO TO 25
      ENDIF

C  Assume M2 and higher multipoles are stretched
      DO 20 I=1,N1
         DO 21 J=1,N2
            IF(L.GT.2*IODD.OR.(L.EQ.2*IODD.AND.JT.EQ.-1)) GOTO 22
            IF(JSPIN(I)-L.GT.KSPIN(J).OR.JSPIN(I)+L.LT.KSPIN(J)) GOTO 21
            GO TO 23
22          CONTINUE
            IF(JSPIN(I)-L.NE.KSPIN(J).AND.JSPIN(I)+L.NE.KSPIN(J))
     $       GOTO 21
23          CONTINUE
            IF(JT.EQ.0.OR.JPAR(I).EQ.' '.OR.KPAR(J).EQ.' ')
     $       GO TO 21
            IF(JT.EQ.1.AND.JPAR(I).NE.KPAR(J)) GO TO 21
            IF(JT.EQ.-1.AND.JPAR(I).EQ.KPAR(J)) GO TO 21
            NMAT=NMAT+1
            GO TO 20
21       CONTINUE
20    CONTINUE

C  Score 0 for partial match, 7 for complete match and -7 for mismatch
25    CONTINUE
      IF(NMAT.EQ.N1) ISCORE=ISCORE+7
      IF(NMAT.EQ.0) ISCORE=ISCORE-7
      RETURN
      END

      subroutine SpSn(str,SkipLevel)
      implicit none
      include 'fconv.inc'
      character*9 str
      integer IGET
      logical SkipLevel

      SkipLevel=.False.
      if(index(str,'SP').gt.0) then
c        Make sure Sp for the correct nucleus is used
         if(NucId.NE.SNucId) then
            write(4,100)' *E*',
     $        ' Incorrect Sp. Add Q card. ',
     $        ' Level skipped except for direct ENSDF/ENSDF2 conversion'
     $       ,'Level=',EL(NLEV),'+Sp' 
100         format(/A,A/4X,A/5X,A,F12.3,A)
            SkipLevel=.true.
            return
         endif
         KRatio=IGET(DSId(1:10))/float(A)
         EL(NLEV)=Sp+KRatio*x(1)
         DEL(NLev)=sqrt(DSp**2+x(2)**2)
         Sep=.true.
      elseif(index(str,'SN').gt.0) then
c        Make sure Sn for the correct nucleus is used
         if(NucId.NE.SNucId) then
            write(4,100)' *E*',
     $        ' Incorrect Sn. Add Q card. ',
     $        ' Level skipped except for direct ENSDF/ENSDF2 conversion'
     $       ,'Level=',EL(NLEV),'+Sn' 
            SkipLevel=.true.
            return 
         endif

         KRatio=IGET(DSId(1:10))/float(A)
         EL(NLEV)=Sn+KRatio*x(1)
         DEL(NLev)=sqrt(DSn**2+x(2)**2)
         Sep=.true.
      endif

      return
      end

      subroutine Skip(INP)
c
c     Skips cards until EOF or End of Data Set
c
      INCLUDE 'fconv.inc'

10    read(INP,'(A)',end=99) card(1)
      if(card(1)(1:10).ne.' ') goto 10
      return

c     End-of-file found
99    TheEnd=.TRUE.

      return
      end

C=================================================================
      
c      SUBROUTINE WAGS(OUT,mode)
c  Writes AGS file using common block data

C=================================================================

      subroutine mul(ll,dum2)
c
c     Get multipolarity of gamma ll for WAGS.
c
      implicit none

      include 'fconv.inc'

      character*(*) dum2

      integer ll

      dum2='  '

c     Get multipolarity from ML and MLPar
      if(ML(1,ll).ge.0) then
         write(dum2(2:2),'(I1)') ML(1,ll)
         if(MLPar(ll).ne.' ') then
            dum2(1:1)='E'
            if(MLPar(ll).eq.'+') then
               if((-1)**ML(1,ll).eq.-1) dum2(1:1)='M'
            elseif(MLPar(ll).eq.'-') then
               if((-1)**ML(1,ll).eq.1) dum2(1:1)='M'
            endif
         endif
      endif

      return
      end

      subroutine label(ll,iband,n,name)
c
c     Get name for level ll for WAGS.
c     ll is level number, n is level number in band, name is
c     the generated label
c
      implicit none

      include 'fconv.inc'

      character*(*) name,dum*5

      integer ll,ispin,n,iband

      dum=Nucid
      call tbsup(dum)
      call padlft(dum,5)
      name=dum

      if(Ban(ll).ne.' ') then
c        Band level, band number iband, level number n
         name(6:6)=BdPi(iband)
         name(7:7)=Bd(iband)
         if(spin(ll).ge.0) then
c           Known spin
            if(JOffs(ll).eq.' ') then
c              No J offset
               ispin=spin(ll)/iodd
               write(name(9:10),104) ispin
104            format(I2.2)
            else
c              With J offset
               ispin=(BdK(iband)+(n-1)*BdStep(iband))/iodd
               if(ispin.ge.0) then
                  write(name(9:10),104) ispin
                  if(ispin.eq.0.and.iodd.eq.2) name(9:10)='-0'  
               else
                  name(9:10)='-1'
               endif
            endif 
         else
c           Unknown spin
            ispin=(BdK(iband)+(n-1)*BdStep(iband))/iodd
            if(ispin.ge.0) then
               write(name(9:10),104) ispin
               if(ispin.eq.0.and.iodd.eq.2) name(9:10)='-0'  
            else
               name(9:10)='-1'
            endif
         endif
         name(8:8)='O'
         if((ispin/2)*2.eq.ispin) name(8:8)='E'
      else
c        Non-band level, level number 0
         name(6:6)=Parity(ll)
         if(name(6:6).eq.' ') name(6:6)='?'
         call charad(ll+151,name(7:8))
         ispin=spin(ll)/iodd
         if(ispin.ge.0) then
            write(name(9:10),104) ispin
            if(ispin.eq.0.and.iodd.eq.2) name(9:10)='-0'  
         else
            name(9:10)='-1'
         endif
      endif

      return
      end

      subroutine LevStat(UncLev,Ispin,parity,JPiMod,JOffs,itent)

c     Determines level uncertainty status variable (itent) for AGS format
c     from UncLev and JPiMod:
c
c      itent     Label in Radware    JPiMod    Ispin  parity
c       0         J Pi                '   '     ge 0   ne ' '
c       1         (J) Pi              ' (J'     ge 0   ne ' '
c       2         J (Pi)              ' (P'     ge 0   ne ' '
c       3         (J Pi)              ' (B'     ge 0   ne ' '
c       4         J                   '   '     ge 0   ' '
c       5         (J)                 ' (J'     ge 0   ' '
c       6         none                any       lt 0   any
c  JPffs ne ' ' gives label 6 
    
      implicit none
      character*1 UncLev,JPiMod*3,parity,JOffs
      integer itent,Ispin

      itent=6
      if(JOffs.ne.' ') then
         itent=6
      elseif(Ispin.lt.0) then
         itent=6
      elseif(Ispin.ge.0.and.parity.eq.' '.and.JPiMod(3:3).eq.'J') then
         itent=5
      elseif(Ispin.ge.0.and.parity.eq.' '.and.JPiMod(3:3).eq.' ') then
         itent=4
      elseif(Ispin.ge.0.and.parity.ne.' '.and.JPiMod(3:3).eq.'B') then
         itent=3
      elseif(Ispin.ge.0.and.parity.ne.' '.and.JPiMod(3:3).eq.'P') then
         itent=2
      elseif(Ispin.ge.0.and.parity.ne.' '.and.JPiMod(3:3).eq.'J') then
         itent=1
      elseif(Ispin.ge.0.and.parity.ne.' '.and.JPiMod(3:3).eq.' ') then
         itent=0
      endif

      return
      end

      subroutine BdProp

c     Determinies band properties (parity, spin step) for a band.
c     The parameters are mainly used for the naming of levels by
c     WAGS.

      implicit none
      integer i,j,ii,counter,lastspin,idiff,iget2
      logical BsOK
      INCLUDE 'fconv.inc'
    
      do j=1,NBand
c        Define default band properties
         BsOK=.false.
         BdPi(j)='?'
         BdK(j)=-1
         BdStep(j)=2*iodd
         counter=0
         lastspin=-99

         do i=1,NLev
            ii=loo(i)
            if(ii.le.0) goto 10
            if(Ban(ii).eq.BD(j)) then
c              Parity of band determined by first parity given
               if(parity(ii).ne.' '.and.BdPi(j).eq.'?') then
                  BdPi(j)=parity(ii)
               endif 
c              BdStep is 1 or 2. Determined by first spin difference.
c              BdK is calculated from this or from spin of first level
               counter=counter+1
               if(counter.eq.1) then
                  ioffs(j)=0
c                 Get J offset if given
                  if(JOffs(ii).ne.' ') then
                     ioffs(j)=iget2(BdStr(j))
                     if(ioffs(j).le.0) ioffs(j)=-1
                  endif
                  if(spin(ii).ge.0) then
                     BdK(j)=spin(ii)+ioffs(j)
                  else
                     BdK(j)=spin(ii)
                  endif
               endif
               if(counter.ge.2) then
                  idiff=(spin(ii)-lastspin)
                  if (idiff.lt.0) idiff = -idiff
                  if((idiff/iodd.eq.1.or.idiff/iodd.eq.2)
     $                                           .and..not.BsOK) then
                     BdStep(j)=idiff
                     if(iodd.eq.2.and.JOffs(ii).ne.' ') then
                        BdStep(j)=BdStep(j)*2
                     endif
                     if(BdK(j).lt.0.and.JOffs(ii).eq.' ')
     $                 BdK(j)=spin(ii)-(counter-1)*BdStep(j)
                     BsOK=.true.
                  endif
               endif
               if(spin(ii).ge.0) lastspin=spin(ii)
            endif 
         enddo
10       continue
      enddo
      return
      end

      subroutine bandcheck
c     Check for duplicate labels
      implicit none
      integer i,j
      INCLUDE 'fconv.inc'
c  Check labels
      do i=1,NLev-1
         do j=i+1,NLev
            if(LevLab(i).eq.LevLab(j)) then
               write(4,100)' *E* Duplicate label= ',LevLab(i),
     $           'Levels= ',EL(i),EL(j) 
100            format(/A,A/5X,A,2F12.3)
            endif
         enddo
      enddo

c  Check parities
      do j=1,NBAND
         do i=1,NLev
            if(Ban(i).eq.BD(j).and.BdPi(j).ne.'?') then
               if(parity(i).ne.' '.and.parity(i).ne.BdPi(j)) then
                  write(4,101)' *E* Parity deviates from band parity',
     $              'Level: ',EL(i),' Parities: ',parity(i),BdPi(j)
101               format(/A/5X,A,F10.2,A,A,1X,A)
               endif
            endif
         enddo
      enddo

      return
      end
