c                                                             FCONV.INC
c  
c     Include file for the program FConv. This file defines common blocks
c     and variables for the program.
c
c     Parameters defining the size of arrays:
      integer MLev,MGam,MBand,MCard

c  FConv variables:
      integer logon
      common /fconv/ logon

c  Dummy variables:
      integer idum
      double precision x(3)
      common/dum1/ idum
      common/dum2/ x
      character dumstr*10
      common/dums/ dumstr
      character sdate*9,stime*8
      common/dat/ sdate,stime

c  Data set variables:
      integer A,Z,IOdd,DSStrL,LComm
      real SP,DSP,SN,DSN,KRatio
      logical Sep
      common/dset/ A,Z,IODD,SP,DSP,SN,DSN,KRatio,Sep,DSStrL,LComm
c   A         Mass number of nucleus
c   Z         Charge of nucleus
c   IODD      =2 for odd nuclei, =1 for even nuclei
c   SP        Proton separation energy
c   DSP       Uncertainty in proton separation energy
c   SN        Neutron separation energy
c   DSN       Uncertainty in neutron separation energy
c   KRatio    Kinematic ratio for reaction (A(target)/A(comp.nucl.)
c   Sep       Separation (Sp or Sn) energy used in data set
c   DSStrL    Length of data set selection string
c   LComm     Length of conversion comment (character Comm(60))

c  Level variables:
      parameter (MLev=3000)
      parameter (MBand=50)
c   MLev      Maximum number of levels
c   MBand     Maximum number of bands
      double precision EL(MLev),DEL(MLev)
      real             T(MLev),DT(MLev),DT2(MLev)
      integer nlev,ISCORE(MLev),L(MLev),LO(MLev),LOO(MLev),NDEC(MLev)
      integer SPIN(MLev),BdStep(MBand),BdK(MBand),ioffs(MBand),Jlev
      common/levels/ NLev,Jlev,EL,DEL,Iscore,L,LO,NDec,Spin,T,DT,
     $               DT2,LOO,BdStep,BdK,ioffs
c   NLev      Number of levels
c   EL        Level energy
c   DEL       Uncertainty in level energy
c   IScore    Score for final level identification. High score is good.
c             ISCORE is a dummy variable used for finding the final level
c             in an ENSDF data set.
c   L         Level card of level i is card number L(i) (for ENSDF input)
c   LO        Array redefining level order
c   LOO       Array listing level numbers in energy order
c   NDec      Number of depopulating gammas from this level
c   Spin      Spin of level. J=2 is spin=2 (IODD=1). J=5/2 is spin=5 (IODD=2).
c             J=-1 is unknown spin
c   T         Half-life in seconds
c   DT        Right uncertainty in half-life
c   DT2       Left uncertainty in half-life
c   BdStep    Delta J in band
c   BdK       Spin of band head
c   ioffs     Spin offset of band
c

c  Gamma variables:
      parameter (MGam=3000)
c   MGam      Maximum number of gammas
      double precision egam(MGam),degam(MGam)
      real    IGam(MGam),DIGam(MGam),D(MGam),DD(MGam),DD2(MGam)
      integer ngam,inlev(MGam),finlev(MGam),ML(2,MGam),Jgam
      common/gammas/ NGam,Jgam,EGam,DEGam,IGam,DIGam,
     $               InLev,FinLev,D,DD,DD2,ML
c   EGam      Gamma-ray energy
c   DEGam     Uncertainty in gamma-ray energy
c   IGam      Gamma relative intensity
c   DIGam     Uncertainty in gamma relative intensity
c   InLev     Initial level of gamma (level number)
c   FinLev    Final level of gamma (level number)
c   D         Multipole mixing ratio
c   DD        Right uncertainty in multipole mixing ratio (+xx)
c   DD2       Left uncertainty in multipole mixing ratio (-xx)
c   ML        Multipolarities of transition (L-values). ML=-1 is unknown

c  Miscellaneous:
      logical TheEnd,NewDS
      integer NBand,NAlpha,NBeta,NPar
      common/misc/ TheEnd,NewDS,NBand,NAlpha,NBeta,NPar
c   TheEnd    True when end of input file is reached
c   NewDS     True when new data set is found
c   NBand     Number of bands
c   NAlpha    Number of alphas
c   NBeta     Number of betas
c   NPar      Number of parents

c  Character common data:
      parameter (MCard=4000)
      character*1 MEl,Ban,BD,CARD*80,parity,DSign,MLPar,BdPi
      character*2 egams,igams,ds,els,ts
      character NUCID*5,DSID*71,BANDID*63,CPAR*5,MULT*10
      character JPIMOD*3,MLMod*3,SNucId*5,DSStr*40,Comm*60
      character*60 verstr,UncLev*1,UncTr*1,JOffs*1,BdStr*18,LevLab*10
      common/char/ MEl(MLev),Ban(MLev),NUCID,DSID,BANDID(MBand),
     $             BD(MBand),CPar(2),Parity(MLev),JPiMod(MLev),
     $             els(MLev),ts(MLev),verstr,UncLev(MLev),JOffs(MLev),
     $             BdStr(MBand),LevLab(MLev),BdPi(MBand)
      common/char1/ Mult(MGam),EGamS(MGam),IGamS(MGam),DS(MGam),
     $              DSign(MGam),MLMod(MGam),MLPar(MGam),SNucid,DSStr,
     $              Comm,UncTr(MGam)
      common/card1/ CARD(MCard)
c   MEL       Letter for +X in level energy
c   Ban       Band marker for level
c   NucId     Nucleus identification (A:1-3, element symbol:4-5)
c   DSId      Data set identification
c   BD        Band identifier
c   CPar      Parent Nucid (for decay)
c   Parity    Parity of level (values '+','-',' ' is unknown parity)
c   JPiMod    Modifier to SPIN and PARITY: ' (J' - spin uncertain,
c             ' (P' - parity uncertain, ' (B' - both uncertain.
c   MLPar     Parity of transition ('+','-',' ' is unknown)
c   ELS       Error string for Ex (contains 'GT','LT', etc.)
c   TS        Error string for half-life (contains 'GT','LT', etc.)
c   verstr    Version number of FConv
c   UncLev    Marker for uncertain levels     
c   JOffs     Offset for spin (e.g. J)
c   BdStr     JPi field of band head
c   LevLab    Level label (for LVL format)
c   BdPi      Parity of band (for labelling)
c   BandID    Band identification string
c   Mult      Multipolarity of gamma (ENSDF string)
c   EGamS     Error string for Egamma (contains 'GT','LT', etc.)
c   IGamS     Error string for Igamma (contains 'GT','LT', etc.)
c   DSign     Sign string for mixing ratio ('+' for positive MR,
c             ' ' for negative MR and for unknown sign)
c   DS        Error string for mixing ratio (contains 'GT','LT', etc.)
c   MLMod     Modifier to multipolarity
c             'xyz', x - first L, y - second L, z - parity. For
c             safe assignments x, y, z are ' ', for uncertain '(',
c             for assumed '[', for incorrect '*'.
c   SNucId    Nucid for which nucleon separation energies are given
c   DSStr     String for selecting data sets
c   Comm      String for exchanging commets between files
c   UncTr     Marker for uncertain transitions
c   Card      Card images of ENSDF data set (for ENSDF <-> ENSDF/2 conv.)
