      SUBROUTINE RAGS(inname)

C     Read Radford's ESCL8R AGS format into common block data arrays

C	Since the radware program and the ENSDF files have slightly different
C	goals, the information they hold is not entirely compatible.  There 
C	are a number of things that are lost in translation.  
C
C	Most important are the various degrees of tentativeness, which should
C	be inserted manually.
C
C	Also, there is a danger when more than one level has the same energy
C	that the reader will not know which level gammas should end on.  This,
C	too, for the time being has a manual solution.
C
C	Roby Austin, September 1998

      
      INTEGER  MASSN
      INTEGER  i,J

      INCLUDE 'GLS.CMN'
      INCLUDE 'fconv.inc'

      INTEGER  GO(MGam), GOO(MGam)

C     For NUCID, the mass number is necessary (we'll check it later)

      WRITE(*,*) 'Enter A'
      READ(*,*) MASSN

      WRITE(NUCID(1:3),'(I3)')MASSN

      IW = 6

      CALL READ_ASCII_GLS

      NewDS = .TRUE.
       
      NBAND = NBANDS
      NLEV = LEVEL
      NGAM = GAMMA
      Z = ATOMIC_NO

      IF(MOD(LEVDATA(1,3)*2,2).EQ.0)THEN
	iodd = 0
      ELSEIF(MOD(LEVDATA(1,3)*2,2).EQ.1)THEN
	iodd = 2
      ELSE
        WRITE(*,*) 'IODD PROBLEM'
	GOTO 99
      ENDIF

C     Check the mass number against the spins

      IF(iodd.EQ.0)THEN
	IF(MOD(MASSN,2).NE.0)THEN
	  WRITE(*,*) 'Mass number does not match spins'
	  GOTO 99
       	ENDIF
      ELSEIF(iodd.EQ.2)THEN
	IF(MOD(MASSN,2).NE.1)THEN
	  WRITE(*,*) 'Mass number does not match spins'
	  GOTO 99
        ENDIF
      ENDIF
      	

C     Put the band information into common blocks

      DO i = 1, NBAND
	BANDID(i) = BAND_NAME(i)
        BD(i) = CHAR(ICHAR('A')-1+i)
      ENDDO

C     Put the level information into common blocks

      DO i = 1, NLEV
	el(i) = LEVDATA(i,1)
	del(i) = LEVDATA(i,2)
	els(i) = ' '
	mel(i) = ' '
	joffs(i) = ' '
	ts(i) = ' ' 
	t(i) = 0.0
	dt(i) = 0.0
	dt2(i) = 0.0	

	ban(i) = CHAR(ICHAR('A')-1+BAND(i))

	IF(L_LABEL_FLAG(i).GT.3)THEN
	  parity(i) = ' '
	ELSEIF(LEVDATA(i,4).EQ.-1.0)THEN
	  parity(i) = '-'
	ELSEIF(LEVDATA(i,4).EQ.1.0)THEN
	  parity(i) = '+'
	ELSE
          WRITE(*,*) 'PARITY PROBLEM'
	  GOTO 99
        ENDIF

c	Read_Ascii_gls.f makes the spins into decimals instead of fractions.  
c	WENSDF doesn't, so we need to correct for that.

	IF(iodd.EQ.2)THEN
	  spin(i) = 2*LEVDATA(i,3)
	ELSE
	  spin(i) = LEVDATA(i,3)
	ENDIF

C	Toggle the tentative spins.  

	IF(L_LABEL_FLAG(i).EQ.0)THEN
	  jpimod(i) = ' '
	ELSEIF(L_LABEL_FLAG(i).EQ.1)THEN
	  jpimod(i) = ' (J'
	ELSEIF(L_LABEL_FLAG(i).EQ.2)THEN
	  jpimod(i) = ' (P'
	ELSEIF(L_LABEL_FLAG(i).EQ.3)THEN
	  jpimod(i) = ' (B'
	ELSEIF(L_LABEL_FLAG(i).EQ.4)THEN
	  jpimod(i) = ' '
	ELSEIF(L_LABEL_FLAG(i).EQ.5)THEN
	  jpimod(i) = ' (J'
	ELSEIF(L_LABEL_FLAG(i).EQ.6)THEN
	  jpimod(i) = ' '
	ENDIF

C	The tentative levels in Radware don't contain as much information as
C	the tentative levels in ensdf.  No account of the energy labels is 
C	taken.  No account of the "thick" level option is taken, either.

	IF(LEVEL_FLAG(i).EQ.1)THEN
	  unclev(i) = '?'
	ELSEIF((LEVEL_FLAG(i).EQ.0).OR.(LEVEL_FLAG(i).EQ.2))THEN
	  unclev(i) = ' '
	ELSE
          WRITE(*,*) 'LEVEL_FLAG PROBLEM'
	  GOTO 99
	ENDIF
			
      ENDDO

c     WENSDF uses this sort routine to write out the levels in order of
c     increasing energy.

      CALL SORT(NLEV,lo,loo,el)

c     The following is a solution to the problem of identical levels.  There
c     will still be difficulties with correctly flagging the gammas into or 
c     out of the levels, though.

      DO I = 1,NLEV - 1
	DO J = I+1, NLEV
	  IF (LO(I).EQ.LO(J)) THEN
	    LOO(LO(J) - 1) = J
	    LO(J) = LO(I) - 1
	  ENDIF
	ENDDO
      ENDDO

      DO J = 1,NGAM
	LOO(J) = 0
	DO I = 1,NGAM
	  IF (LO(I).EQ.J) THEN
	    LOO(J) = I
	    GOTO 10
	  ENDIF
	ENDDO
10	CONTINUE
      ENDDO


c     Now move on to the gammas.

      DO i = 1,NGAM
        egam(i) = GAMDATA(i,1)
      ENDDO

c     The gamms must appear in order of increasing energy.

      CALL SORT(NGAM,go,goo,egam)

c     This is a solution to the problem of identical gammas.  

      DO I = 1,NGAM-1
        DO J = I+1,NGAM
          IF (GO(I).EQ.GO(J)) THEN
	    GOO(GO(J)-1) = J
	    GO(J) = GO(I) - 1
	  ENDIF 
        ENDDO
      ENDDO

      DO J = 1,NGAM
	GOO(J) = 0
	DO I = 1,NGAM
	  IF (GO(I).EQ.J) THEN
	    GOO(J) = I
	    GOTO 20
	  ENDIF
	ENDDO
20	CONTINUE
      ENDDO
      

C     Put the gamma information into common blocks

      DO i = 1,NGAM
	inlev(i) = GAMID(goo(i),1)
	finlev(i) = GAMID(goo(i),2)
	egam(i) = GAMDATA(goo(i),1)
	degam(i) = GAMDATA(goo(i),2)
	igam(i) = GAMDATA(goo(i),3)
	digam(i) = GAMDATA(goo(i),4)
	ML(1,i) = GAMN(goo(i))
 	d(i) = GAMDATA(goo(i),7)
 	dd(i) = GAMDATA(goo(i),9)
	dd2(i) = 0
	ML(2,i) = 0
	egams(i) = ' '
	igams(i) = ' '
	ds(i) = ' '
	DSign(i) = ' '
	MLMod(i) = '   '	

c	IF (GAMMA_FLAG(GOO(I)).EQ.1) THEN 
c	  UNCTR(I) = '?'
c	ELSE
c	  UNCTR(I) = ' '
c	ENDIF


C	The tentative gammas in Radware don't contain as much information as 
C	the tentative gammas in ENSDF.  No account is taken of the labels.

	IF(GAMMA_FLAG(goo(i)).EQ.1)THEN
	  UncTr(i) = '?'
	ELSEIF(GAMMA_FLAG(goo(i)).EQ.0)THEN
	  UncTr(i) = ' '
	ELSE
          WRITE(*,*) 'GAMMA_FLAG PROBLEM'
          WRITE(*,*) GAMMA_FLAG(GOO(I))
	  GOTO 99
	ENDIF

	IF(GAML(goo(i)).EQ.'E')THEN
	  MLPar(i) = '+'
	  IF(MOD(GAMN(goo(i)),2).EQ.1) MLPar(i) = '-'
	ELSE
	  MLPar(i) = '+'
	  IF(MOD(GAMN(goo(i)),2).EQ.0) MLPar(i) = '-'
	ENDIF

	IF(d(i).NE.0.0)THEN
	  ML(2,i) = ML(1,i) + 1
	ENDIF

      ENDDO

      TheEnd = .TRUE.

      return

99    write(*,*) 'Line 99 goto from RAGS.  Problem'
      return

      end
